package jp.co.nikkeibp.nsw;
import java.util.Random;

public class Minesweeper {
	static final private int BOMB = -1;
	static final private int OPENED = 1;
	static final private int BORDER = -2;
	private int table_x, table_y, bomb_num;
	private int opened_cell = 0, open_cell_num = -1;
	static final private int[] scan_x = {-1, 0, 1, -1, 1, -1, 0, 1}; 
	static final private int[] scan_y = {-1, -1, -1, 0, 0, 1, 1, 1};
	private int[][] table_n, table_v, table_m;
	private boolean isGame_over = false;

	Minesweeper(int _table_x, int _table_y) {
		table_x = _table_x + 2;
		table_y = _table_y + 2;
		table_n = new int[table_y][table_x];
		table_v = new int[table_y][table_x];
		table_m = new int[table_y][table_x];
		
		for(int x=0; x<table_x; x++) {
			table_n[0][x] = BORDER;
			table_n[table_y-1][x] = BORDER;
			table_v[0][x] = OPENED;
			table_v[table_y-1][x] = OPENED;
			table_m[0][x] = BORDER;
			table_m[table_y-1][x] = BORDER;
		}
		for(int y=0; y<table_y; y++) {
			table_n[y][0] = BORDER;
			table_n[y][table_x-1] = BORDER;
			table_v[y][0] = OPENED;
			table_v[y][table_x-1] = OPENED;
			table_m[y][0] = BORDER;
			table_m[y][table_x-1] = BORDER;
		}
	}

	public void clearTable() {      
		isGame_over = false;
		opened_cell = 0;
		open_cell_num = -1;
		    
		for(int x=1; x<table_x-1; x++) {
			for(int y=1; y<table_y-1; y++) {
				table_n[y][x] = 0;
				table_v[y][x] = 0;
				table_m[y][x] = 0;
			}
		}
	}
	
	public void setBomb(int first_x, int first_y, int difficulty) {
		bomb_num = (int) (table_x - 2) * (table_y - 2) / difficulty;	
		open_cell_num = (table_x - 2) * (table_y - 2) - bomb_num;
		first_x++;
		first_y++;
		Random r = new Random(System.currentTimeMillis());
		for(int i=0; i<bomb_num; i++) {
			int rx = r.nextInt(table_x - 2) + 1;
			int ry = r.nextInt(table_y - 2) + 1;
			if(rx == first_x && ry == first_y) { 
				i--;
				continue;
			}
			if(table_n[ry][rx] != BOMB) {
				table_n[ry][rx] = BOMB;
			}else if(table_n[ry][rx] == BOMB) {
				i--; 
			}
		}
		for(int x=1; x<table_x - 1; x++) {
			for(int y=1; y<table_y - 1; y++) {
				int adjacent_bomb = 0;
				if(table_n[y][x]!=BOMB) {
					for(int i=0; i<8; i++){
						if(table_n[y+scan_y[i]][x+scan_x[i]]==BOMB) { adjacent_bomb++; }
					}      
			        table_n[y][x] = adjacent_bomb;
				}
			}
		}	 
	}

	public int getCellNumber(int x, int y) {		
		return table_n[y+1][x+1];
	}
	
	public boolean isCellOpen(int x, int y) {
		return table_v[y+1][x+1] == OPENED ? true : false;
	}
	
	public boolean openCellmkII(int x, int y) {
		x++; y++;
		if(table_n[y][x] > 0 && table_v[y][x] == OPENED){
			int num = 0;
			for(int i=0; i<8; i++){
				if(table_m[y+scan_y[i]][x+scan_x[i]] == 1) { num++; }			
			}
			if(num != table_n[y][x]) {
				return false;
			}
			
			for(int i=0; i<8; i++){
				if(table_n[y+scan_y[i]][x+scan_x[i]]==0){
					openBlank(x+scan_x[i], y+scan_y[i]);
				}else if(table_n[y+scan_y[i]][x+scan_x[i]]!=BOMB && table_v[y+scan_y[i]][x+scan_x[i]]!=OPENED) {
					table_v[y+scan_y[i]][x+scan_x[i]] = OPENED;
					opened_cell++;
				}else if(table_n[y+scan_y[i]][x+scan_x[i]]==BOMB){
					if(table_m[y+scan_y[i]][x+scan_x[i]] != 1) {
						isGame_over = true;
					}
				}
			}
		}
		openNumCell();
		
		return true;
	}	
	
	public void openCell(int x, int y) {
		x++; y++;
		_openCell(x, y);
	}
	
	private void _openCell(int x, int y) {
		if(table_n[y][x] == 0) {
			openBlank(x, y);
			openNumCell();
		}else if(table_n[y][x] > 0 && table_v[y][x] != OPENED){
			table_v[y][x] = OPENED;
			opened_cell++;
		}else if(table_n[y][x] == BOMB){
			isGame_over = true; 
		}
	}
	
	private void openBlank(int x, int y) {
		  if(table_n[y][x] == 0 && table_v[y][x] != OPENED) {
		    table_v[y][x] = OPENED;
		    opened_cell++;
		  }
		  for(int i=0; i<8; i++){
		  	if(table_n[y+scan_y[i]][x+scan_x[i]] == 0 && table_v[y+scan_y[i]][x+scan_x[i]] != OPENED) {
		  		openBlank(x+scan_x[i], y+scan_y[i]); 
		  	}
		  }
	}

	private void openNumCell() {
		for(int x=1; x<table_x - 1; x++) {
			for(int y=1; y<table_y - 1; y++) {
				boolean open_flag = false;
				if(table_v[y][x] != OPENED) {
					for(int i=0; i<8; i++){  
						if(table_n[y+scan_y[i]][x+scan_x[i]] == 0 && table_v[y+scan_y[i]][x+scan_x[i]] == OPENED) {
							open_flag = true; 
						}
					}
				}
				if(open_flag) { 
					table_v[y][x] = OPENED; 
					opened_cell++;
				}
			}
		}
	}
	
	public boolean isGameover() {
		return isGame_over;
	}
	
	public boolean isClear() {
		return opened_cell == open_cell_num ? true : false;
	}
	
	public void flipMark(int x, int y) {
		if(table_v[y+1][x+1] != OPENED) {
			table_m[y+1][x+1] = 1 - table_m[y+1][x+1];
		}
	}
		
	public boolean isMark(int x, int y) {
		return table_m[y+1][x+1] == 1 ? true : false;
	}
}








