package jp.co.nikkeibp.nsw;

import android.content.Context;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.view.GestureDetector;
import android.view.MotionEvent;
import android.view.View;

public class MinesweeperView extends View implements GestureDetector.OnGestureListener {
	static final private int CELL = 48;
	static final private int TABLE_X = 10;
	static final private int TABLE_Y = 15;
	static public int difficulty = 0;
	
	private GestureDetector gesturedetector = new GestureDetector(this);
	private Minesweeper msweeper;
	private Bitmap[] panel = new Bitmap[12];
	private boolean isFirstopen = true;
	private boolean isGame_over = false;
	private boolean isClear = false;
	private boolean isLongpressed = false;
	
	public MinesweeperView(Context context) {
		super(context);
		// TODO Auto-generated constructor stub
		msweeper = new Minesweeper(TABLE_X, TABLE_Y);
		
		Resources r = getResources();
		panel[0] = BitmapFactory.decodeResource(r, R.drawable.p0);
		panel[1] = BitmapFactory.decodeResource(r, R.drawable.p1);
		panel[2] = BitmapFactory.decodeResource(r, R.drawable.p2);
		panel[3] = BitmapFactory.decodeResource(r, R.drawable.p3);
		panel[4] = BitmapFactory.decodeResource(r, R.drawable.p4);
		panel[5] = BitmapFactory.decodeResource(r, R.drawable.p5);
		panel[6] = BitmapFactory.decodeResource(r, R.drawable.p6);
		panel[7] = BitmapFactory.decodeResource(r, R.drawable.p7);
		panel[8] = BitmapFactory.decodeResource(r, R.drawable.p8);
		panel[9] = BitmapFactory.decodeResource(r, R.drawable.mark);
		panel[10] = BitmapFactory.decodeResource(r, R.drawable.mine);
		panel[11] = BitmapFactory.decodeResource(r, R.drawable.unopened);
	}

	public void reset(){
		msweeper.clearTable();
		isFirstopen = true;
		isGame_over = false;
		isClear = false;
		isLongpressed = false;
	}
	
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);
        
        for(int x=0; x<TABLE_X; x++) {
            for(int y=0; y<TABLE_Y; y++) {
            	if(msweeper.isCellOpen(x, y) == false){
            		if(msweeper.isMark(x, y)){
            			canvas.drawBitmap(panel[9], x*CELL, y*CELL, null);
            		}else{
            			canvas.drawBitmap(panel[11], x*CELL, y*CELL, null);
            		}
            	}else{
            		int num = msweeper.getCellNumber(x, y);
            		canvas.drawBitmap(panel[num], x*CELL, y*CELL, null);
            	}
            }
        }
        if(isGame_over){
            for(int x=0; x<TABLE_X; x++) {
                for(int y=0; y<TABLE_Y; y++) {
                	int num = msweeper.getCellNumber(x, y);
                	if(num == -1) {
                		canvas.drawBitmap(panel[10], x*CELL, y*CELL, null);
                	}
                }
            }
        }
    }
    
    public boolean onTouchEvent(MotionEvent event) {
    	if(gesturedetector.onTouchEvent(event)) {
    		return true;
    	}

   		switch(event.getAction()){
   		case MotionEvent.ACTION_UP:		
   			int x = (int) (event.getX() / CELL);
   			int y = (int) (event.getY() / CELL);
   			
   			if(x > TABLE_X || y > TABLE_Y) {
   				return true;
   			}
   			
   			if(isFirstopen) {
   				msweeper.setBomb(x, y, difficulty);
   				isFirstopen = false;
   			}
   			
   			if(isLongpressed){
   				msweeper.flipMark(x, y);	
   				
   				if(msweeper.openCellmkII(x, y)==false){
   					MinesweeperActivity.vibrate(100);
   				}
   		   		isLongpressed = false;
   			}else{  			
   				msweeper.openCell(x, y);
   			}
   			break;
   		}
		isGame_over = msweeper.isGameover();
   		isClear = msweeper.isClear();
   		if(isGame_over) {
   			MinesweeperActivity.gameOver();
   		}else if(isClear) {
   			MinesweeperActivity.gameClear();
   		}
   		invalidate();
        return true;
    }
    
	public boolean onDown(MotionEvent e) {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean onFling(MotionEvent e1, MotionEvent e2, float velocityX,
			float velocityY) {
		// TODO Auto-generated method stub
		return false;
	}

	public void onLongPress(MotionEvent e) {
		// TODO Auto-generated method stub
   		isLongpressed = true;
	}

	public boolean onScroll(MotionEvent e1, MotionEvent e2, float distanceX,
			float distanceY) {
		// TODO Auto-generated method stub
		return false;
	}

	public void onShowPress(MotionEvent e) {
		// TODO Auto-generated method stub
		
	}

	public boolean onSingleTapUp(MotionEvent e) {
		// TODO Auto-generated method stub
		return false;
	}
}
