//
//  Minesweeper.m
//  Minesweeper
//
//  Created by KenKen on 11/03/17.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "Minesweeper.h"

const int scan_x[] = {-1, 0, 1, -1, 1, -1, 0, 1}; 
const int scan_y[] = {-1, -1, -1, 0, 0, 1, 1, 1};


@implementation Minesweeper


- (id)initWithTableX:(int)_table_x TableY:(int)_table_y
{
    if(self == [super init]) {
        table_x = _table_x + 2;
        table_y = _table_y + 2;
        isGame_over = false;
        opened_cell = 0;
        open_cell_num = -1;

        table_n = new int*[table_y];
        table_v = new int*[table_y];
        table_m = new int*[table_y];
        for (int i=0; i<table_y; i++) {
            table_n[i] = new int[table_x];
            table_v[i] = new int[table_x];
            table_m[i] = new int[table_x];
        }
        
        for(int x=0; x<table_x; x++) {
            for(int y=0; y<table_y; y++) {
                table_n[y][x] = 0;
                table_v[y][x] = 0;
                table_m[y][x] = 0;
            }
        }
      
        for(int x=0; x<table_x; x++) {
            table_n[0][x] = BORDER;
            table_n[table_y-1][x] = BORDER;
            table_v[0][x] = OPENED;
            table_v[table_y-1][x] = OPENED;
            table_m[0][x] = BORDER;
            table_m[table_y-1][x] = BORDER;
        }
        for(int y=0; y<table_y; y++) {
            table_n[y][0] = BORDER;
            table_n[y][table_x-1] = BORDER;
            table_v[y][0] = OPENED;
            table_v[y][table_x-1] = OPENED;
            table_m[y][0] = BORDER;
            table_m[y][table_x-1] = BORDER;
        }
    }
    
    return self;
}

- (void)clearTable
{        
    isGame_over = false;
    opened_cell = 0;
    open_cell_num = -1;
    
    for(int x=1; x<table_x-1; x++) {
        for(int y=1; y<table_y-1; y++) {
            table_n[y][x] = 0;
            table_v[y][x] = 0;
            table_m[y][x] = 0;
        }
    }
}

- (void)dealloc
{
    for(int i=0; i<table_y; i++){
        delete[] table_n[i]; 
        delete[] table_v[i];
        delete[] table_m[i];
    }
    delete[] table_n;
    delete[] table_v;
    delete[] table_m;
    
    [super dealloc];
}

- (void)setBombFirstX:(int)first_x FirstY:(int)first_y Difficulty:(int)difficulty;
{    
    bomb_num = (int)(table_x-2) * (table_y-2) / difficulty;
    open_cell_num = (table_x-2) * (table_y-2) - bomb_num;
    first_x++;
    first_y++;
    srand(time(NULL));
    
    for(int i=0; i<bomb_num; i++) {
        int rx = rand() % (table_x - 2) + 1;
        int ry = rand() % (table_y - 2) + 1;
        if(rx == first_x && ry == first_y) { 
            i--;
            continue;
        }
        if(table_n[ry][rx] != BOMB) {
            table_n[ry][rx] = BOMB;
        }else if(table_n[ry][rx] == BOMB) {
            i--; 
        }
    }
    for(int x=1; x<table_x - 1; x++) {
        for(int y=1; y<table_y - 1; y++) {
            int adjacent_bomb = 0;
            if(table_n[y][x]!=BOMB) {
                for(int i=0; i<8; i++){
                    if(table_n[y+scan_y[i]][x+scan_x[i]]==BOMB) { adjacent_bomb++; }
                }
                table_n[y][x] = adjacent_bomb;
            }
        }
    }
}

- (int)getCellNumberX:(int)x Y:(int)y 
{
    return table_n[y+1][x+1];
}

- (bool)isCellOpenX:(int)x Y:(int)y 
{
    return table_v[y+1][x+1] == OPENED ? true : false;

}

- (bool)openCellmkIIX:(int)x Y:(int)y
{
    x++;
    y++;
    if(table_n[y][x] > 0 && table_v[y][x] == OPENED){
        int num = 0;
        for(int i=0; i<8; i++){
            if(table_m[y+scan_y[i]][x+scan_x[i]] == 1) { num++; }			
        }
        if(num!=table_n[y][x]) {
            return false;
        }
        
        for(int i=0; i<8; i++){
            if(table_n[y+scan_y[i]][x+scan_x[i]]==0){
                [self openBlankX:x+scan_x[i] Y:y+scan_y[i]];
            }else if(table_n[y+scan_y[i]][x+scan_x[i]]!=BOMB && table_v[y+scan_y[i]][x+scan_x[i]]!=OPENED) {
                table_v[y+scan_y[i]][x+scan_x[i]] = OPENED;
                opened_cell++;
            }else if(table_n[y+scan_y[i]][x+scan_x[i]]==BOMB){
                if(table_m[y+scan_y[i]][x+scan_x[i]] != 1) {
                    isGame_over = true;
                }
            }
        }
    }    
    [self openNumCell];
    
    return true;
}

- (void)openCellX:(int)x Y:(int)y
{
    x++;
    y++;
    [self _openCellX:x Y:y];
}

- (bool)isGameover
{
    return isGame_over;
}

- (bool)isClear
{    
    return opened_cell == open_cell_num ? true : false; 
}

- (void)flipMarkX:(int)x Y:(int)y
{
    if(table_v[y+1][x+1] != OPENED) {
        table_m[y+1][x+1] = 1 - table_m[y+1][x+1];
    }  
}

- (bool)isMarkX:(int)x Y:(int)y
{
    return table_m[y+1][x+1] == 1 ? true : false;     
}


- (void)_openCellX:(int)x Y:(int)y
{
    if(table_n[y][x] == 0) {
        [self openBlankX:x Y:y];
        [self openNumCell];
    }else if(table_n[y][x] > 0 && table_v[y][x] != OPENED){
        table_v[y][x] = OPENED;
        opened_cell++;
    }else if(table_n[y][x] == BOMB){
        isGame_over = true; 
    }       
}

- (void)openBlankX:(int)x Y:(int)y
{
    if(table_n[y][x] == 0 && table_v[y][x] != OPENED) {
        table_v[y][x] = OPENED;
        opened_cell++;
    }
    for(int i=0; i<8; i++){
        if(table_n[y+scan_y[i]][x+scan_x[i]] == 0 && table_v[y+scan_y[i]][x+scan_x[i]] != OPENED) {
            [self openBlankX:x+scan_x[i] Y:y+scan_y[i]];
        }
    } 
}

- (void)openNumCell
{
    for(int x=1; x<table_x - 1; x++) {
        for(int y=1; y<table_y - 1; y++) {
            bool open_flag = false;
            if(table_v[y][x] != OPENED) {
                for(int i=0; i<8; i++){  
                    if(table_n[y+scan_y[i]][x+scan_x[i]] == 0 && table_v[y+scan_y[i]][x+scan_x[i]] == OPENED) {
                        open_flag = true; 
                    }
                }
            }
            if(open_flag) { 
                table_v[y][x] = OPENED; 
                opened_cell++;
            }
        }
    }
}


@end
