//
//  MinesweeperView.m
//  Minesweeper
//
//  Created by KenKen on 11/03/17.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "MinesweeperView.h"


@implementation MinesweeperView

@synthesize msweeper;

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        // Initialization code
        app = (MinesweeperAppDelegate *)[[UIApplication sharedApplication] delegate];
        NSString *path;
        path = [[NSBundle mainBundle] pathForResource:@"p0" ofType:@"png"];
        panel[0] = [[UIImage alloc] initWithContentsOfFile:path];
        path = [[NSBundle mainBundle] pathForResource:@"p1" ofType:@"png"];
        panel[1] = [[UIImage alloc] initWithContentsOfFile:path];
        path = [[NSBundle mainBundle] pathForResource:@"p2" ofType:@"png"];
        panel[2] = [[UIImage alloc] initWithContentsOfFile:path];
        path = [[NSBundle mainBundle] pathForResource:@"p3" ofType:@"png"];
        panel[3] = [[UIImage alloc] initWithContentsOfFile:path];
        path = [[NSBundle mainBundle] pathForResource:@"p4" ofType:@"png"];
        panel[4] = [[UIImage alloc] initWithContentsOfFile:path];
        path = [[NSBundle mainBundle] pathForResource:@"p5" ofType:@"png"];
        panel[5] = [[UIImage alloc] initWithContentsOfFile:path];
        path = [[NSBundle mainBundle] pathForResource:@"p6" ofType:@"png"];
        panel[6] = [[UIImage alloc] initWithContentsOfFile:path];
        path = [[NSBundle mainBundle] pathForResource:@"p7" ofType:@"png"];
        panel[7] = [[UIImage alloc] initWithContentsOfFile:path];
        path = [[NSBundle mainBundle] pathForResource:@"p8" ofType:@"png"];
        panel[8] = [[UIImage alloc] initWithContentsOfFile:path];
        path = [[NSBundle mainBundle] pathForResource:@"mark" ofType:@"png"];
        panel[9] = [[UIImage alloc] initWithContentsOfFile:path];
        path = [[NSBundle mainBundle] pathForResource:@"mine" ofType:@"png"];
        panel[10] = [[UIImage alloc] initWithContentsOfFile:path];
        path = [[NSBundle mainBundle] pathForResource:@"unopened" ofType:@"png"];
        panel[11] = [[UIImage alloc] initWithContentsOfFile:path];
        
        msweeper = [[Minesweeper alloc] initWithTableX:TABLE_X TableY:TABLE_Y];
        isFirstopen = true;
        isGame_over = false;
        isClear = false;
        
        tap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(onTouchEvent:)];
        [self addGestureRecognizer:tap];
        longpress = [[UILongPressGestureRecognizer alloc] initWithTarget:self action:@selector(onLongPress:)];
        [self addGestureRecognizer:longpress];
        
        sound_url = CFBundleCopyResourceURL(CFBundleGetMainBundle(), CFSTR("bomb"), CFSTR("aif"), NULL);
        AudioServicesCreateSystemSoundID(sound_url, &soundID);
        CFRelease(sound_url);
    }
    return self;
}

- (void)reset
{
    [msweeper clearTable];
    isFirstopen = true;
    isGame_over = false;
    isClear = false;
}


// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect
{   
    // Drawing code
    for(int x=0; x<TABLE_X; x++) {
        for(int y=0; y<TABLE_Y; y++) {
            if([msweeper isCellOpenX:x Y:y] == false){
                if([msweeper isMarkX:x Y:y]){
                    [panel[9] drawAtPoint:CGPointMake(x*CELL, y*CELL)];
                }else{
                    [panel[11] drawAtPoint:CGPointMake(x*CELL, y*CELL)];
                }
            }else{
                int num = [msweeper getCellNumberX:x Y:y];
                [panel[num] drawAtPoint:CGPointMake(x*CELL, y*CELL)];
            }
        }
    }
    if(isGame_over){
        for(int x=0; x<TABLE_X; x++) {
            for(int y=0; y<TABLE_Y; y++) {
                int num = [msweeper getCellNumberX:x Y:y];
                if(num == -1) {
                    [panel[10] drawAtPoint:CGPointMake(x*CELL, y*CELL)];
                }
            }
        }
    }
}

- (bool)onTouchEvent:(UITapGestureRecognizer *)recognizer 
{
    CGPoint location = [recognizer locationInView:self];
    int x = location.x / CELL;
    int y = location.y / CELL;
    if(x > TABLE_X || y > TABLE_Y) {
        return false;
    }  
    if(isFirstopen) {
        [msweeper setBombFirstX:x FirstY:y Difficulty:app.difficulty];
        isFirstopen = false;
    }
    
    [msweeper openCellX:x Y:y];
    
    isGame_over = [msweeper isGameover];
    isClear = [msweeper isClear];
    if (isGame_over) {
        [self gameOver];
    }else if (isClear) {
        [self gameClear];
    }    
    [self setNeedsDisplay];

    return true;
}

- (bool)onLongPress:(UILongPressGestureRecognizer *)recognizer 
{
    if(recognizer.state != UIGestureRecognizerStateEnded){
        return false;
    }
        
    CGPoint location = [recognizer locationInView:self];
    int x = location.x / CELL;
    int y = location.y / CELL;
    if(x > TABLE_X || y > TABLE_Y) {
        return false;
    }
    
    [msweeper flipMarkX:x Y:y];
    
    if([msweeper openCellmkIIX:x Y:y]==false){
        // バイブレーションの処理
        AudioServicesPlaySystemSound(kSystemSoundID_Vibrate);
    }
    isGame_over = [msweeper isGameover];
    isClear = [msweeper isClear];
    if (isGame_over) {
        [self gameOver];
    }else if (isClear) {
        [self gameClear];
    }
    
    [self setNeedsDisplay];

    return true;
}

- (void)gameOver
{
    AudioServicesPlaySystemSound(soundID);
    app.alertview = [[[UIAlertView alloc] initWithTitle:@"ゲームオーバー" 
                                               message:@"" 
                                              delegate:app.msViewController
                                     cancelButtonTitle:nil
                                     otherButtonTitles:@"メニューに戻る", nil] autorelease];
    [app.alertview show];
    AudioServicesPlaySystemSound(kSystemSoundID_Vibrate);
}

- (void)gameClear
{
    app.end_time = CFAbsoluteTimeGetCurrent();
    NSString *s = @"クリア！";
    int time = (int)(app.end_time - app.start_time);

    app.alertview = [[[UIAlertView alloc] initWithTitle:[s stringByAppendingFormat:@"%d秒でクリアです", time]
                                                                          message:@"" 
                                                                         delegate:app.msViewController
                                                                cancelButtonTitle:nil
                                                                otherButtonTitles:@"メニューに戻る", nil] autorelease];
    [app.alertview show];
}


- (void)dealloc
{
    [panel[0] release];
    [panel[1] release];
    [panel[2] release];
    [panel[3] release];
    [panel[4] release];
    [panel[5] release];
    [panel[6] release];
    [panel[7] release];
    [panel[8] release];
    [panel[9] release];
    [panel[10] release];
    [panel[11] release];
    [msweeper release];
    [tap release];
    [longpress release];
    AudioServicesDisposeSystemSoundID(soundID);  
    
    [super dealloc];
    
}

@end
